package ru.dbotthepony.kstarbound.defs.world

import com.google.common.collect.ImmutableMap
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.collect.WeightedList
import ru.dbotthepony.kstarbound.json.builder.JsonFactory

@JsonFactory
data class SpawnerConfig(
	val spawnCellSize: Int,
	val spawnCellMinimumEmptyTiles: Int,
	val spawnCellMinimumLiquidTiles: Int,
	val spawnCellMinimumNearSurfaceTiles: Int,
	val spawnCellMinimumNearCeilingTiles: Int,
	val spawnCellMinimumAirTiles: Int,
	val spawnCellMinimumExposedTiles: Int,
	val spawnCellNearSurfaceDistance: Int,
	val spawnCellNearCeilingDistance: Int,

	val minimumDayLevel: Double,
	val minimumLiquidLevel: Double,
	val spawnCheckResolution: Double,
	val spawnSurfaceCheckDistance: Int,
	val spawnCeilingCheckDistance: Int,
	val spawnProhibitedCheckPadding: Double,

	val spawnCellLifetime: Double,
	val windowActivationBorder: Int,

	val defaultActive: Boolean = true,
	val debug: Boolean = false,

	val spawnGroups: ImmutableMap<String, WeightedList<Registry.Ref<SpawnType>>> = ImmutableMap.of(),
) {
	init {
		require(spawnCellSize >= 1) { "Bad spawnCellSize: $spawnCellSize" }
		require(spawnCellMinimumEmptyTiles >= 0) { "Negative spawnCellMinimumEmptyTiles: $spawnCellMinimumEmptyTiles" }
		require(spawnCellMinimumLiquidTiles >= 0) { "Negative spawnCellMinimumLiquidTiles: $spawnCellMinimumLiquidTiles" }
		require(spawnCellMinimumNearSurfaceTiles >= 0) { "Negative spawnCellMinimumNearSurfaceTiles: $spawnCellMinimumNearSurfaceTiles" }
		require(spawnCellMinimumNearCeilingTiles >= 0) { "Negative spawnCellMinimumNearCeilingTiles: $spawnCellMinimumNearCeilingTiles" }
		require(spawnCellMinimumAirTiles >= 0) { "Negative spawnCellMinimumAirTiles: $spawnCellMinimumAirTiles" }
		require(spawnCellMinimumExposedTiles >= 0) { "Negative spawnCellMinimumExposedTiles: $spawnCellMinimumExposedTiles" }
		require(spawnCellNearSurfaceDistance >= 0) { "Negative spawnCellNearSurfaceDistance: $spawnCellNearSurfaceDistance" }
		require(spawnCellNearCeilingDistance >= 0) { "Negative spawnCellNearCeilingDistance: $spawnCellNearCeilingDistance" }

		require(windowActivationBorder >= 0) { "Negative windowActivationBorder: $windowActivationBorder" }
	}
}
